import requests
import json
import os
import time

# originally inspired by https://github.com/chandulal/airflow-testing/blob/master/src/integrationtest/python/airflow_api.py
if not "AIRFLOW_HOST" in os.environ:
    raise Exception("AIRFLOW_HOST is not set")
AIRFLOW_HOST = os.environ["AIRFLOW_HOST"]

if not "AIRFLOW_PORT" in os.environ:
    raise Exception("AIRFLOW_PORT is not set")
AIRFLOW_PORT = os.environ["AIRFLOW_PORT"]        

class AirflowAPI:
    def __init__(self):
        # wait for API to come up
        max_cnt = 60
        cnt = 0
        status = -1
        while cnt < max_cnt and status != 200:
            try:
                poke = requests.get("{url}/api/experimental/test".format(url=self.get_airflow_url()), timeout=0.1)
                status = poke.status_code
            except:
                print("Caught exception waiting for airflow REST API")
            cnt += 1
            if cnt < max_cnt and status != 200:
                print("Waiting for airflow webserver at {url} (status={code})".format(url=self.get_airflow_url(),code=status))
                time.sleep(2.0)
        if status != 200:
            raise Exception("we waited for airflow webserver but status={status} (not 200)".format(status=status))


    def get_airflow_url(self):
        return "http://{host}:{port}".format(host=AIRFLOW_HOST, port=AIRFLOW_PORT)


    def trigger_dag(self, dag_id, execution_date):
        print("triggering dag={dag_id} on {ed}".format(dag_id=dag_id,ed=execution_date))
        payload = {"execution_date" : execution_date}
        response = requests.post("{url}/api/experimental/dags/{dag_id}/dag_runs".format(url=self.get_airflow_url(),dag_id=dag_id), json=payload)
        if response.status_code != 200:
            raise Exception("trigger POST request returned {code}".format(code=response.status_code))
    
    def dag_state(self, dag_id, execution_date):
        responses = requests.get("{url}/api/experimental/dags/{dag_id}/dag_runs".format(url=self.get_airflow_url(),dag_id=dag_id)).json()
        for response in responses:
            if response["execution_date"] == execution_date:
                return response
        raise Exception("Failed to find execution_date={dt} in responses={r}".format(dt=execution_date,r=responses))

    def is_dag_running(self, dag_id, execution_date):
        response = self.dag_state(dag_id, execution_date)
        return response["state"] == "running"

    def get_dag_status(self, dag_id, execution_date):
        response = self.dag_state(dag_id, execution_date)
        return response["state"]
