from airflow import DAG
from airflow.operators.subdag_operator import SubDagOperator
from airflow.operators.python_operator import PythonOperator

import os
import pathlib
from datetime import datetime, timedelta
import pendulum
import _gojsonnet    


CONFIG_ROOT = os.environ.get("CONFIG_ROOT","/usr/local/airflow/test-output")
pathlib.Path(CONFIG_ROOT).mkdir(parents=True, exist_ok=True)
JSONNET_ROOT=os.path.join(pathlib.Path(__file__).parent.parent.absolute(),"jsonnet")


## runs jsonnet
def eval_jsonnet(**kwargs):
    jsonnet_file = kwargs['jsonnet_file']
    json_file = kwargs['json_file']
    json_str = _gojsonnet.evaluate_file(jsonnet_file)
    
    with open(json_file,"w") as json_output:
        print(json_str, file=json_output)
    print("wrote {} successfully", json_file)

    
def get_json_cfg(test_name,run_id):
    config_root = os.path.join(CONFIG_ROOT, test_name)
    pathlib.Path(config_root).mkdir(parents=True, exist_ok=True)
    return os.path.join(config_root,"{test_name}.{run_id}.json".format(test_name=test_name,run_id=run_id))


def subdag(parent_name, child_name, default_args, schedule_interval, user_defined_macros, run_id, test_name):
    dag = DAG('{}.{}'.format(parent_name,child_name),
              default_args=default_args,
              schedule_interval=schedule_interval,
              is_paused_upon_creation=None,
              user_defined_macros=user_defined_macros
    )

    json_cfg = get_json_cfg(test_name, run_id)
    
    test_cfg = PythonOperator(task_id='generate_json',
                              provide_context=True,
                              python_callable=eval_jsonnet,
                              op_kwargs={'jsonnet_file' : os.path.join(JSONNET_ROOT,"{test_name}.jsonnet".format(test_name=test_name)),
                                         'json_file' : json_cfg,
                              },
                              dag=dag)
    
    return dag


local_tz = pendulum.timezone("America/Los_Angeles")

default_args = {
    'owner': 'Airflow',
    'depends_on_past': False,
    'start_date': datetime(2020, 2, 1, tzinfo=local_tz),
    'email_on_failure': False,
    'email_on_retry': False,
    'retries': 1,
    'retry_delay': timedelta(minutes=5),
}

# individual 
dag = DAG('test-dag',
          default_args=default_args,
          schedule_interval=None,
          is_paused_upon_creation=None,
          user_defined_macros={},
)

test1_name = "test1"
test1 = SubDagOperator(
    subdag=subdag(dag.dag_id,
                  test1_name,
                  default_args,
                  dag.schedule_interval,
                  dag.user_defined_macros,
                  "{{ run_id }}",
                  test1_name),
    task_id=test1_name,
    default_args=default_args,
    dag=dag
)
